#version 330

precision highp float; // Video card drivers require this line to function properly

in vec2 uvCoord;

uniform sampler2D sceneSampler;
uniform sampler2D depthTexture;
//uniform sampler2D noiseTexture;
//uniform int noiseRand;

uniform mat4 viewProjectionInverseMatrix;
uniform mat4 previousViewProjectionMatrix;

out vec4 fragColor;

void main(void)
{
	// Get the depth buffer value at this pixel.  
	float zOverW = texture(depthTexture, uvCoord).r;
	// H is the viewport position at this pixel in the range -1 to 1.
	vec4 H = vec4(uvCoord.x * 2 - 1, (1 - uvCoord.y) * 2 - 1, zOverW, 1);
	// Transform by the view-projection inverse.
	vec4 D = viewProjectionInverseMatrix * H;
	// Divide by w to get the world position.
	vec4 worldPos = D / D.w;

	// Current viewport position
	vec4 currentPos = H;  
	// Use the world position, and transform by the previous view-projection matrix.
	vec4 previousPos = previousViewProjectionMatrix * worldPos;
	// Convert to nonhomogeneous points [-1,1] by dividing by w.
	previousPos /= previousPos.w;
	// Use this frame's position and last frame's to compute the pixel velocity.
	vec2 velocity = (currentPos.xy - previousPos.xy)/2.f;

	// PERFORM MOTION BLUR
	int numSamples   = 10;			// 1 sample means no motion blur at all
	float blurFactor = 0.025;		// how much blur do you want?
	
	vec2 coord = uvCoord;
	velocity = (velocity * blurFactor) / numSamples ;
	
	// Get the initial color at this pixel.
	vec4 color = texture(sceneSampler, coord);
	coord += velocity;

	for(int i = 1; i < numSamples; ++i, coord += velocity)
	{
		// Sample the color buffer along the velocity vector.
		vec4 currentColor = texture(sceneSampler, coord);
		// Add the current color to our color sum.
		color += currentColor;
	}  
	// Average all of the samples to get the final blur color.
	
	fragColor = color / numSamples;

	//// apply some hacky NOISE
	//float noiseAmount = 0.06; // 7% of noise look good
	//float noiseSize = 0.27;
	//vec2 noiseLookup = (uvCoord + noiseRand/100.0) / noiseSize;
	//vec3 noiseValue = texture(noiseTexture, noiseLookup).rgb;
	//fragColor = vec4 ( (fragColor.rgb + noiseValue.g*noiseAmount) / (1.0), 1.0);
}
